diff --git a/src/libFLAC/stream_encoder_intrin_avx2.c b/src/libFLAC/stream_encoder_intrin_avx2.c
index 4381bdb..d0cd0e4 100644
--- a/src/libFLAC/stream_encoder_intrin_avx2.c
+++ b/src/libFLAC/stream_encoder_intrin_avx2.c
@@ -77,8 +77,7 @@ void FLAC__precompute_partition_info_sums_intrin_avx2(const FLAC__int32 residual
 				}
 
 				for( ; residual_sample < end; residual_sample++) {
-					__m128i res128 = _mm_cvtsi32_si128(residual[residual_sample]);
-					res128 = _mm_abs_epi32(res128);
+					__m128i res128 = _mm_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					sum128 = _mm_add_epi32(sum128, res128);
 				}
 
@@ -102,15 +101,13 @@ void FLAC__precompute_partition_info_sums_intrin_avx2(const FLAC__int32 residual
 				sum128 = _mm_add_epi64(_mm256_extracti128_si256(sum256, 1), _mm256_castsi256_si128(sum256));
 
 				for( ; (int)residual_sample < (int)end-1; residual_sample+=2) {
-					__m128i res128 = _mm_loadl_epi64((const __m128i*)(residual+residual_sample));
-					res128 = _mm_abs_epi32(res128);
+					__m128i res128 = _mm_abs_epi32(_mm_loadl_epi64((const __m128i*)(residual+residual_sample)));
 					res128 = _mm_cvtepu32_epi64(res128);
 					sum128 = _mm_add_epi64(sum128, res128);
 				}
 
 				for( ; residual_sample < end; residual_sample++) {
-					__m128i res128 = _mm_cvtsi32_si128(residual[residual_sample]);
-					res128 = _mm_abs_epi32(res128);
+					__m128i res128 = _mm_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					sum128 = _mm_add_epi64(sum128, res128);
 				}
 
diff --git a/src/libFLAC/stream_encoder_intrin_sse2.c b/src/libFLAC/stream_encoder_intrin_sse2.c
index be8395a..ed01978 100644
--- a/src/libFLAC/stream_encoder_intrin_sse2.c
+++ b/src/libFLAC/stream_encoder_intrin_sse2.c
@@ -43,6 +43,17 @@
 #include <stdlib.h>    /* for abs() */
 #include <emmintrin.h> /* SSE2 */
 #include "FLAC/assert.h"
+#include "share/compat.h"
+
+FLAC__SSE_TARGET("sse2")
+static inline __m128i local_abs_epi32(__m128i val)
+{
+	__m128i mask = _mm_srai_epi32(val, 31);
+	val = _mm_xor_si128(val, mask);
+	val = _mm_sub_epi32(val, mask);
+	return val;
+}
+
 
 FLAC__SSE_TARGET("sse2")
 void FLAC__precompute_partition_info_sums_intrin_sse2(const FLAC__int32 residual[], FLAC__uint64 abs_residual_partition_sums[],
@@ -70,26 +81,17 @@ void FLAC__precompute_partition_info_sums_intrin_sse2(const FLAC__int32 residual
 
 				/* assumption: residual[] is properly aligned so (residual + e1) is properly aligned too and _mm_loadu_si128() is fast */
 				for( ; residual_sample < e1; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
-					__m128i mm_mask = _mm_srai_epi32(mm_res, 31);
-					mm_res = _mm_xor_si128(mm_res, mm_mask);
-					mm_res = _mm_sub_epi32(mm_res, mm_mask); /* abs(INT_MIN) is undefined, but if the residual is INT_MIN we have bigger problems */
+					__m128i mm_res = local_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < e3; residual_sample+=4) {
-					__m128i mm_res = _mm_loadu_si128((const __m128i*)(residual+residual_sample));
-					__m128i mm_mask = _mm_srai_epi32(mm_res, 31);
-					mm_res = _mm_xor_si128(mm_res, mm_mask);
-					mm_res = _mm_sub_epi32(mm_res, mm_mask);
+					__m128i mm_res = local_abs_epi32(_mm_loadu_si128((const __m128i*)(residual+residual_sample)));
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < end; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
-					__m128i mm_mask = _mm_srai_epi32(mm_res, 31);
-					mm_res = _mm_xor_si128(mm_res, mm_mask);
-					mm_res = _mm_sub_epi32(mm_res, mm_mask);
+					__m128i mm_res = local_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
@@ -108,27 +110,18 @@ void FLAC__precompute_partition_info_sums_intrin_sse2(const FLAC__int32 residual
 				FLAC__ASSERT(e1 <= end);
 
 				for( ; residual_sample < e1; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]); /*  0   0   0   r0 */
-					__m128i mm_mask = _mm_srai_epi32(mm_res, 31);
-					mm_res = _mm_xor_si128(mm_res, mm_mask);
-					mm_res = _mm_sub_epi32(mm_res, mm_mask); /*  0   0   0  |r0|  ==   00   |r0_64| */
+					__m128i mm_res = local_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample])); /*  0   0   0  |r0|  ==   00   |r0_64| */
 					mm_sum = _mm_add_epi64(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < e3; residual_sample+=2) {
-					__m128i mm_res = _mm_loadl_epi64((const __m128i*)(residual+residual_sample)); /*  0   0   r1  r0 */
-					__m128i mm_mask = _mm_srai_epi32(mm_res, 31);
-					mm_res = _mm_xor_si128(mm_res, mm_mask);
-					mm_res = _mm_sub_epi32(mm_res, mm_mask); /*  0   0  |r1|   |r0| */
+					__m128i mm_res = local_abs_epi32(_mm_loadl_epi64((const __m128i*)(residual+residual_sample))); /*  0   0  |r1|   |r0| */
 					mm_res = _mm_shuffle_epi32(mm_res, _MM_SHUFFLE(3,1,2,0)); /* 0  |r1|  0  |r0|  ==  |r1_64|  |r0_64|  */
 					mm_sum = _mm_add_epi64(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < end; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
-					__m128i mm_mask = _mm_srai_epi32(mm_res, 31);
-					mm_res = _mm_xor_si128(mm_res, mm_mask);
-					mm_res = _mm_sub_epi32(mm_res, mm_mask);
+					__m128i mm_res = local_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					mm_sum = _mm_add_epi64(mm_sum, mm_res);
 				}
 
diff --git a/src/libFLAC/stream_encoder_intrin_ssse3.c b/src/libFLAC/stream_encoder_intrin_ssse3.c
index 547ef95..465950b 100644
--- a/src/libFLAC/stream_encoder_intrin_ssse3.c
+++ b/src/libFLAC/stream_encoder_intrin_ssse3.c
@@ -70,20 +70,17 @@ void FLAC__precompute_partition_info_sums_intrin_ssse3(const FLAC__int32 residua
 
 				/* assumption: residual[] is properly aligned so (residual + e1) is properly aligned too and _mm_loadu_si128() is fast */
 				for( ; residual_sample < e1; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
-					mm_res = _mm_abs_epi32(mm_res); /* abs(INT_MIN) is undefined, but if the residual is INT_MIN we have bigger problems */
+					__m128i mm_res = _mm_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < e3; residual_sample+=4) {
-					__m128i mm_res = _mm_loadu_si128((const __m128i*)(residual+residual_sample));
-					mm_res = _mm_abs_epi32(mm_res);
+					__m128i mm_res = _mm_abs_epi32(_mm_loadu_si128((const __m128i*)(residual+residual_sample)));
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < end; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
-					mm_res = _mm_abs_epi32(mm_res);
+					__m128i mm_res = _mm_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
@@ -102,21 +99,18 @@ void FLAC__precompute_partition_info_sums_intrin_ssse3(const FLAC__int32 residua
 				FLAC__ASSERT(e1 <= end);
 
 				for( ; residual_sample < e1; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]); /*  0   0   0   r0 */
-					mm_res = _mm_abs_epi32(mm_res); /*  0   0   0  |r0|  ==   00   |r0_64| */
+					__m128i mm_res = _mm_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample])); /*  0   0   0  |r0|  ==   00   |r0_64| */
 					mm_sum = _mm_add_epi64(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < e3; residual_sample+=2) {
-					__m128i mm_res = _mm_loadl_epi64((const __m128i*)(residual+residual_sample)); /*  0   0   r1  r0 */
-					mm_res = _mm_abs_epi32(mm_res); /*  0   0  |r1|   |r0| */
+					__m128i mm_res = _mm_abs_epi32(_mm_loadl_epi64((const __m128i*)(residual+residual_sample))); /*  0   0  |r1|   |r0| */
 					mm_res = _mm_shuffle_epi32(mm_res, _MM_SHUFFLE(3,1,2,0)); /* 0  |r1|  0  |r0|  ==  |r1_64|  |r0_64|  */
 					mm_sum = _mm_add_epi64(mm_sum, mm_res);
 				}
 
 				for( ; residual_sample < end; residual_sample++) {
-					__m128i mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
-					mm_res = _mm_abs_epi32(mm_res);
+					__m128i mm_res = _mm_abs_epi32(_mm_cvtsi32_si128(residual[residual_sample]));
 					mm_sum = _mm_add_epi64(mm_sum, mm_res);
 				}
 
